/*  CardString.java -- JavaCard version of Java String class, constant

    This is part of SecureFW project, protection interface for which allows controled usage of secured code.
    See http://www.svenda.com/petr/dpthesis/index.html for more information
    Copyright (C) 2004  Petr Svenda <petr@svenda.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please, report any bugs to author <petr@svenda.com>
/**/

package com.svenda.SecureAlg;
import javacard.framework.*;

//
//  CARD STRING
//
public class CardString {
  public short m_end = 0;            // POINTER TO FIRST FREE ITEM
  public byte  m_buffer[];

  protected CardString() {
  }

  public CardString(byte[] buffer) {
    m_end = (short) buffer.length;
    m_buffer = buffer;
  }

  public short length() {
    return m_end;
  }

  public short getBuffer(byte[] returnBuffer, short offset, short bufferMaxLength) throws ArrayIndexOutOfBoundsException {
    short result = 0;
    if ((short) (offset + m_end) > bufferMaxLength) {
      // TARGET ARRAY TO SMALL
      throw new ArrayIndexOutOfBoundsException();
    }
    else {
      Util.arrayCopy(m_buffer, (short) 0, returnBuffer, offset, m_end);
      result = m_end;
    }

    return result;
  }

  public byte[] getBuffer() {
    byte[]  buffer = new byte[m_end];

    Util.arrayCopy(m_buffer, (short) 0, buffer, (short) 0, m_end);

    return buffer;
  }

   public byte compare(byte[] buffer, short offset, short count) {
    byte  result = 0;
    short diff = 0;

    diff = (short) (m_end - count);
    if (diff < 0) result = -1;
    else {
      if (diff > 0) result = 1;
      else result = Util.arrayCompare(buffer, offset, m_buffer, (short) 0, count);
    }

    return result;
  }

  public boolean endsWith(byte[] buffer, short offset, short count) {
    boolean  result = false;

    if (count > m_end) {
      // NO
    }
    else {
      if (Util.arrayCompare(buffer, offset, m_buffer, (short) (m_end - count), count) == 0) result = true;
    }

    return result;
  }

  public boolean startsWith(byte with) {
    if (m_end > 0 && m_buffer[0] == with) return true;
    else return false;
  }

  public static boolean convertFromAsciiDigits(byte buffer[], short bufferOffset, short bufferLength) {
    short   i;
    short   hlp;
    boolean status = true;

    hlp = (short) (bufferOffset + bufferLength);
    for (i = bufferOffset; i < hlp; i++) {
      switch (buffer[i]) {
        case 0x30: buffer[i] = 0x00; break; // 0
        case 0x31: buffer[i] = 0x01; break; // 1
        case 0x32: buffer[i] = 0x02; break; // 2
        case 0x33: buffer[i] = 0x03; break; // 3
        case 0x34: buffer[i] = 0x04; break; // 4
        case 0x35: buffer[i] = 0x05; break; // 5
        case 0x36: buffer[i] = 0x06; break; // 6
        case 0x37: buffer[i] = 0x07; break; // 7
        case 0x38: buffer[i] = 0x08; break; // 8
        case 0x39: buffer[i] = 0x09; break; // 9
        default: {
          // NOT ASCII DIGIT
          status = false;
        }
      }
    }

    return status;
  }

  public short getShortNumber() throws ArithmeticException {
    short number = 0;
    short i;
    short exp = 1;

    if (convertFromAsciiDigits(m_buffer, (short) 0, m_end)) {
      for (i = (short) (m_end - 1); i >= 0; i--) {
        number += m_buffer[i] * exp;
        exp *= 0x0A;
      }
    }
    else {
      throw new ArithmeticException();
    }

    return number;
  }

  public static boolean isWhitespace(byte b) {
    if (b == 9) return true;  //HORIZONTAL TABULATION.
    if (b == 10) return true; //LINE FEED.
    if (b == 11) return true; //VERTICAL TABULATION.
    if (b == 12) return true; //FORM FEED.
    if (b == 13) return true; //CARRIAGE RETURN.
    if (b == 28) return true; //FILE SEPARATOR.
    if (b == 29) return true; //GROUP SEPARATOR.
    if (b == 30) return true; //RECORD SEPARATOR.
    if (b == 31) return true; //UNIT SEPARATOR.
    if (b == 32) return true; //SPACE
    return false; // NO WHITESPACE CHARACTER MATCH 'b'
  }
}
