/*  CardStringBuffer.java -- JavaCard version of Java String class, modifiable

    This is part of SecureFW project, protection interface for which allows controled usage of secured code.
    See http://www.svenda.com/petr/dpthesis/index.html for more information
    Copyright (C) 2004  Petr Svenda <petr@svenda.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please, report any bugs to author <petr@svenda.com>
/**/

package com.svenda.SecureAlg;
import javacard.framework.*;

//
//  CARD STRING BUFFER
//
public class CardStringBuffer extends CardString {
  private short m_allocatedSize = 0;  // actual size of allocated buffer
  private byte  m_increaseStep = 10;  // step used for incresing buffer length
  private static final short DEFAULT_INITIAL_SIZE = 100;

  public CardStringBuffer() {
    initializeBuffer(DEFAULT_INITIAL_SIZE);
  }

  public CardStringBuffer(short initialBufferSize) {
    initializeBuffer(initialBufferSize);
  }

  /**
   * Truncate length of buffer and clear rest to 0x00
   * @param length
   */
  public void setLength(short length) {
    m_end = length;
    for (short i = m_end; i < m_allocatedSize; i++) m_buffer[i] = 0x00;
  }

  /**
   * Append item at the end of buffer
   * @param item
   */
  public void append(byte item) {
    ensureProperlySize((short)(m_end + 1));
    m_buffer[m_end] = item;
    m_end++;
  }

  private void initializeBuffer(short initialBufferSize) {
    m_allocatedSize = initialBufferSize;
    m_buffer = new byte[m_allocatedSize];
  }

  /**
   * Ensure, than CardStringBuffer can contain characters of total length 'expectedLength'
   * @param expectedLength
   */
  private void ensureProperlySize(short expectedLength) {
    if (m_allocatedSize < expectedLength) {
      // RESIZE BUFFER
      m_allocatedSize += m_increaseStep;
      byte[] newBuffer = new byte[m_allocatedSize];
      Util.arrayCopy(m_buffer, (short) 0, newBuffer, (short) 0, m_end);
      m_buffer = newBuffer;
    }
    else {
      // BUFFER SIZE IS ENOUGHT
    }
  }
}
