/*  GaussEaster.java -- example of SecureAlgProfile class implementation, calculate Easter Sunday for given year

    This is part of SecureFW project, protection interface for which allows controled usage of secured code.
    See http://www.svenda.com/petr/dpthesis/index.html for more information
    Copyright (C) 2004  Petr Svenda <petr@svenda.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please, report any bugs to author <petr@svenda.com>
/**/

package com.svenda.SecureAlg;
import javacard.framework.*;

public class GaussEaster extends SecureAlgProfile {

  public GaussEaster(byte dataBuffer[], short offset, short dataLen) {
    super(dataBuffer, offset, dataLen);
  }

  /**
   * Process GAUSS Easter algorithm on given year
   * @param dataBuffer
   * @param offset
   * @param dataLen
   * @return output data length (2 bytes)
   */
  public short ProcessAlgData(byte dataBuffer[], short offset, short dataLen) {
    short   day;
    short   year;
    short   help1;
    short   help2;
    short   help3;

    // CHECK INPUT BUFFER EXPECTED LENGTH
    if (dataLen < 2) ISOException.throwIt(SecureAlgManager.SW_ALG_INPUT_BAD);

    // CHECK USAGE COUNTER AND DECREASE
    if (m_allowedUsageCounter > 0) m_allowedUsageCounter--;
    else ISOException.throwIt(SecureAlgManager.SW_USAGE_NOT_ALLOWED);

    // YEAR IS EXPECTED AS SHORT VALUE
    year = Util.getShort(dataBuffer, offset);


    // GAUSS EASTER ALGORITHM (VALID FOR 20th AND 21th CENTURY)
    // help1 = ((19 * (year % 19) + 24) % 30);
    help1 = (short) (year % 19);
    help1 *= 19;
    help1 += 24;
    help1 %= 30;

    // day = (22 + help1 + ((5 + 2 * (year % 4) + 4 * (year % 7) + 6 * help1) % 7));
    help2 = (short) (year % 4);
    help2 *= 2;
    help2 += 5;
    help3 = (short) (year % 7);
    help3 *= 4;
    help3 += (short) (6 * help1);
    help2 += help3;
    help2 %= 7;
    day = (short) (help2 + help1);
    day += 22;

    if (day >= 57) day -= 7;

    if (day > 31) {
      // APRIL
      day -= 31;
      dataBuffer[offset] = 0x04;
      dataBuffer[(short) (offset + 1)] = (byte) day;
    }
    else {
      // MARCH
      dataBuffer[offset] = 0x03;
      dataBuffer[(short) (offset + 1)] = (byte) day;
    }

    return 2; // MONTH + DAY
  }
}