/*  JavaCardAES.java -- implementation of AES algorithm in JavaCard environment
    NOTE: only for demonstration purposes, no speed optimalization is used !!!

    This is part of SecureFW project, protection interface for which allows controled usage of secured code.
    See http://www.svenda.com/petr/dpthesis/index.html for more information
    Copyright (C) 2004  Petr Svenda <petr@svenda.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please, report any bugs to author <petr@svenda.com>
/**/

package com.svenda.SecureAlg;
import javacard.framework.*;

public class JavaCardAES {
    // NOTE: BLOCKN & KEYN CONSTANTS ARE DEFINED
    // ONLY FOR BETTER READIBILITY OF CODE AND CANNOT BE CHANGED!!!
    final static byte BLOCKLEN                  = (byte) (128 / 8);
    final static byte BLOCKN    		= (byte) (128 / 32);
    final static byte KEYN    		        = (byte) (128 / 32);
    final static byte STATELEN                  = (byte) (4 * BLOCKN);
    final static byte N_ROUNDS    		= (byte) 10;

    final static byte rcon[] = {(byte) 0x01, (byte) 0x02, (byte) 0x04, (byte) 0x08, (byte) 0x10, (byte) 0x20, (byte) 0x40, (byte) 0x80, (byte) 0x1b, (byte) 0x36};

    // shifts[0..3] -> ENCRYPT, shifts[4..7] ... DECRYPT
    final static byte shifts[] = { 0, 1, 2, 3, 0, 3, 2, 1};

    // NOTE: NEXT ARRAYS COULD BE DECLARED STATIC, BUT UNKNOWN PROBLEM OCCURES
    // DURING APPLET INSTALATION ON Gemplus GXPPro-R3.
    private byte SBox[] = null;
    private byte SiBox[] = null;
    private byte Alogtable[] = null;
    private byte Logtable[] = null;

    // SCHEDULED ROUND KEYS
    private byte roundKeys[] = null;

    // PREALOCATED REUSED TRANSIENT BUFFER
    private byte tempBuffer[] = null;

    // INICIALIZATION VECTOR
    public byte       m_IV[] = null;
    public short      m_IVOffset = 0;

    private static short TAUB(byte a) {
      // RETURN SHORT VALUE CONSTRUCTED FROM SIGNED REPRESENTATION OF UNSIGNED VALUE
      // EXAMPLE: byte val = (byte) 250; // val == -6
      //          ASSERT(TAUB(val) == 250);
      return ((a >= 0) ? a : (short) (256 + a));
    }

    public JavaCardAES() {
      // ALLOCATE AND COMPUTE LOOKUP TABLES
      SBox = new byte[256];
      SiBox = new byte[256];
      Alogtable = new byte[256];
      Logtable = new byte[256];
      roundKeys = new byte[(N_ROUNDS + 1) * 4 * KEYN];  // 160 B TOTAL
      tempBuffer = JCSystem.makeTransientByteArray(STATELEN, JCSystem.CLEAR_ON_DESELECT);
      MakeSBox();
    }

    // CALCULATION OF LOOKUP TABLES FOR REDUCING CODE SIZE
    private void MakeSBox() {
      byte   p = 1;
      short  q;
      short  i;

      // Alogtable AND Logtable TABLES
      for (i=0; i<256; ++i) {
          Alogtable[i]= (byte) p;
          Logtable[TAUB(p)]= (byte) i;
          p=(byte) (p^(p<<1)^(((p&0x80) == 0) ? 0: 0x01b));
      }
      // CORRECTION OF GENERATED LOG TABLE IS NEEDED
      Logtable[1] = 0;

      // SBox AND SiBox TABLES
      for (i=0; i<256; ++i)  {
         p=( (i == 0) ? 0 : (Alogtable[(short) (255-TAUB(Logtable[i]))]));
         q = TAUB(p);
         q= (short) ((q>>7) | (q<<1)); p^= (byte) q;
         q= (short) ((q>>7) | (q<<1)); p^= (byte) q;
         q= (short) ((q>>7) | (q<<1)); p^= (byte) q;
         q= (short) ((q>>7) | (q<<1)); p^= (byte) q;
         p= (byte) (p^0x63);
         SBox[i] = (byte) p;
         SiBox[TAUB(p)] = (byte) i;
      }
    }


    // MULTIPLY TWO ELEMENTS OF GF(2^m)
    private byte mul(byte a, byte b) {
      // BODY OF THIS FUNCTION IS BAD LOOKING ONLY BECAUSE OF USING byte TYPE FOR STORING VALUES FROM 0 TO 255
      // IT SIMPLY MEANS COMMAND Alogtable[(Logtable[a] + Logtable[b])%255];
      if ((a != 0)  && (b != 0)) return (byte) Alogtable[TAUB((byte) ((short) (((short) (TAUB(Logtable[TAUB(a)]) + TAUB(Logtable[TAUB(b)]))) % 255)))];
      else return 0;
    }

    // ADD ROUND KEY USING XOR
    private static void KeyAddition(byte a[], short dataOffset, byte rk[], short keyOffset) {
      byte i;
      for (i = 0; i < STATELEN; i++) a[(short) (i + dataOffset)] ^= rk[(short) (i + keyOffset)];
    }

    // SHIFTING ROWS
    private void ShiftRow(byte a[], short dataOffset, byte d) {
      byte i, j;
      // ALSO FIRST ROUND IS SHIFTED (BUT BY 0 POSITIONS) DUE TO POSSIBILITY FOR USING Util.arrayCopy() LATER
      // tempBuffer WILL CONTAINS SHIFTED STATE a
      for(i = 0; i < 4; i++) {
          for(j = 0; j < BLOCKN; j++) tempBuffer[(short) (i + j * 4)] = a[(short) (((i + (byte) ((j + shifts[TAUB((byte) (i + d*4))] % BLOCKN) * 4)) % STATELEN) + dataOffset)];
      }
      Util.arrayCopyNonAtomic(tempBuffer, (short) 0, a, dataOffset, STATELEN);
    }

    // SBox OR SiBox SUBSTITUTION
    private static void Substitution(byte a[], short dataOffset, byte box[]) {
      byte i;
      for (i = 0; i < STATELEN; i++) a[(short) (i + dataOffset)] = box[TAUB(a[(short) (i + dataOffset)])] ;
    }

    // MIXING COLUMNS
    private void MixColumn(byte a[], short dataOffset) {
      byte  i = 0, j = 0;
      // hlp CONTAINS PRECALCULATED EXPRESSION ((j * 4) + dataOffset)
      short hlp = dataOffset;
      // hlp2 CONTAINS PRECALCULATED EXPRESSION (j * 4)
      byte hlp2 = -4;

      hlp -= 4;
      for(j = 0; j < BLOCKN; j++) {
        // TIME REDUCING PRECALCULATION
        hlp += 4; hlp2 += 4;
        for(i = 0; i < 4; i++) {
          tempBuffer[(byte) (i + hlp2)] = (byte) mul((byte) 2, a[(short) (i + hlp)]);
          tempBuffer[(byte) (i + hlp2)] ^= (byte) mul((byte) 3, a[(short) (((i + 1) % 4) + hlp)]);
          tempBuffer[(byte) (i + hlp2)] ^= (byte) a[(short) (((i + 2) % 4) + hlp)];
          tempBuffer[(byte) (i + hlp2)] ^= (byte) a[(short) (((i + 3) % 4) + hlp)];
        }
      }

      Util.arrayCopyNonAtomic(tempBuffer, (short) 0, a, dataOffset, STATELEN);
    }

    // INVERSE OF MIXING COLUMNS
    private void InvMixColumn(byte a[], short dataOffset) {
      byte i = 0, j = 0;
      // hlp CONTAINS PRECALCULATED EXPRESSION ((j * 4) + dataOffset)
      short hlp = dataOffset;
      // hlp2 CONTAINS PRECALCULATED EXPRESSION (j * 4)
      byte hlp2 = -4;

      hlp -= 4;
      for(j = 0; j < BLOCKN; j++) {
        // TIME REDUCING PRECALCULATION
        hlp += 4; hlp2 += 4;
        for(i = 0; i < 4; i++) {
          tempBuffer[(byte) (i + hlp2)] = (byte) mul((byte) 0xe, a[(short) (i + hlp)]);
          tempBuffer[(byte) (i + hlp2)] ^= (byte) mul((byte) 0xb, a[(short) (((i + 1) % 4) + hlp)]);
          tempBuffer[(byte) (i + hlp2)] ^= (byte) mul((byte) 0xd, a[(short) (((i + 2) % 4) + hlp)]);
          tempBuffer[(byte) (i + hlp2)] ^= (byte) mul((byte) 0x9, a[(short) (((i + 3) % 4) + hlp)]);
        }
      }

      Util.arrayCopyNonAtomic(tempBuffer, (short) 0, a, dataOffset, STATELEN);
    }

    /**
     * Sechedule AES round keys fro given key material
     * @param key
     * @param keyOffset
     */
    public void RoundKeysSchedule(byte key[], short keyOffset) {
      byte     i;
      byte     j;
      byte     round;
      byte     rconpointer = 0;
      short    sourceOffset = 0;
      short    targetOffset = 0;
      // hlp CONTAINS PRECALCULATED EXPRESSION (round * (4 * KEYN))
      short    hlp = 0;

      // FIRST KEY (SAME AS INPUT KEY)
      Util.arrayCopyNonAtomic(key, keyOffset, roundKeys, (short) 0, STATELEN);

      // 10 ROUNDS KEYS
      for (round = 1; round <= N_ROUNDS; round++) {
          // TIME REDUCING PRECALCULATION
          hlp += STATELEN;

          // COPY KEY FOR round - 1 TO BUFFER FOR round
          Util.arrayCopyNonAtomic(roundKeys, (short) ((round - 1) * STATELEN), roundKeys, hlp, STATELEN);

          rconpointer = (byte) (round - 1);

          for (i = 0; i < 4; i++) {
            sourceOffset = (short) ( ((i + 1) % 4) + ((KEYN-1) * 4) + hlp );
            targetOffset = (short) ( i + (0 * 4) + hlp );
            roundKeys[targetOffset] ^= SBox[TAUB(roundKeys[sourceOffset])];
          }

          roundKeys[hlp] ^= rcon[rconpointer];

          for (j = 1; j < KEYN; j++) {
              for (i = 0; i < 4; i++) {
                sourceOffset = (short) (i + ((j - 1) * 4) + hlp);
                targetOffset = (short) ((i + (j * 4)) + hlp);
                roundKeys[targetOffset] ^= roundKeys[sourceOffset];
              }
          }
      }
    }

     /**
      * Encrypt one block, key schedule must be already processed
      * @param data
      * @param dataOffset
      * @return true if encrypt success, false otherwise.
      */
     public boolean AESEncryptBlock(byte data[], short dataOffset) {
        byte r;

        KeyAddition(data, dataOffset, roundKeys, (byte) 0);

        // N_ROUNDS-1 ORDINARY ROUNDS
        for(r = 1; r < N_ROUNDS; r++) {
            Substitution(data, dataOffset, SBox);
            ShiftRow(data, dataOffset, (byte) 0);
            MixColumn(data, dataOffset);

            KeyAddition(data, dataOffset, roundKeys, (short) (r * STATELEN));
        }

        // NO MIXCOLUMN IN LAST ROUND
        Substitution(data, dataOffset, SBox);
        ShiftRow(data, dataOffset, (byte) 0);
        KeyAddition(data, dataOffset, roundKeys, (short) (N_ROUNDS * STATELEN));

        return true;
     }

     /**
      * Decrypt one block, key schedule must be already processed
      * @param data
      * @param dataOffset
      * @return true if decrypt success, false otherwise.
      */
     public boolean AESDecryptBlock(byte data[], short dataOffset) {
        byte  r;

        KeyAddition(data, dataOffset, roundKeys, (short) (N_ROUNDS * STATELEN));
        ShiftRow(data, dataOffset, (byte) 1);
        Substitution(data, dataOffset, SiBox);

        for(r = N_ROUNDS-1; r > 0; r--) {
            KeyAddition(data, dataOffset, roundKeys, (short) (r * STATELEN));
            InvMixColumn(data, dataOffset);
            ShiftRow(data, dataOffset, (byte) 1);
            Substitution(data, dataOffset, SiBox);
        }

        KeyAddition(data, dataOffset, roundKeys, (byte) 0);

        return true;
     }


     /**
      * Encrypt buffer using CBC cipher mode, key schedule must be already processed
      * @param data
      * @param dataOffset
      * @param dataLen
      * @param bUsePadding - if TRUE, PKCS#5 padding is used, NO padding otherwise
      * @param bStrengthLastBlock - of TRUE, Vaudenay CBC-PADDING attack countermeasure is used
      * @return output data length
      */
     public short AESEncryptCBC(byte data[], short dataOffset, short dataLen, boolean bUsePadding, boolean bStrengthLastBlock) {
        byte  r;
        short i;
        byte  j;
        byte  bytesToPad = 0;
        short numBlocks = 0;
        short hlp;
        byte  prevBlock[] = new byte[BLOCKLEN];

        if (m_IV == null) ISOException.throwIt(SecureAlgManager.SW_IV_BAD);

        //
        //  INSERT PADDING
        //
        if (bUsePadding) {
          // PKCS#5 PADDING METHOD
          if ((bytesToPad = (byte) (dataLen % BLOCKLEN)) != 0) {
            bytesToPad = (byte) (BLOCKLEN - bytesToPad);

            // CHECK FOR ENOUGHT BUFFER LENGTH
            if (data.length < (short) (dataOffset + dataLen + bytesToPad)) ISOException.throwIt(SecureAlgManager.SW_CIPHER_DATA_LENGTH_BAD);

            // SET LAST bytesToPad BYTES ONTO VALUE 'bytesToPad'
            Util.arrayFillNonAtomic(data, (short) (dataOffset + dataLen), bytesToPad, bytesToPad);
            // INCREASE DATA LENGTH FOR ENCRYPT
            dataLen += bytesToPad;
          }
          else {
            // CHECK FOR ENOUGHT BUFFER LENGTH
            if (data.length < (short) (dataOffset + dataLen + BLOCKLEN)) ISOException.throwIt(SecureAlgManager.SW_CIPHER_DATA_LENGTH_BAD);

            // ADD NEW 16B BLOCK WITH ZEROS
            Util.arrayFillNonAtomic(data, (short) (dataOffset + dataLen), BLOCKLEN, (byte) 0);
            // INCREASE DATA LENGTH FOR ENCRYPT
            dataLen += BLOCKLEN;
          }
        }
        else {
            // NO PADDING USED, MUST BE BLOCKLEN MULTIPLY
            if ((dataLen % BLOCKLEN) != 0) ISOException.throwIt(SecureAlgManager.SW_CIPHER_DATA_LENGTH_BAD);
        }

        hlp = (short) (dataOffset - BLOCKLEN);
        numBlocks = (short) (dataLen / BLOCKLEN);

        // XOR INIT VECTOR
        for (j = 0; j < BLOCKLEN; j++) data[(short) (j + dataOffset)] ^= m_IV[(short) (j + m_IVOffset)];

        //
        // PROCESS ALL BLOCKS EXCEPT LAST
        //
        for (i = 0; i < (short) (numBlocks - 1); i++) {
            hlp += BLOCKLEN;
            // XOR PREVIOUS CIPHER OUTPUT
            if (i > 0) for (j = 0; j < BLOCKLEN; j++) data[(short) (j + hlp)] ^= data[(short) (j + hlp - BLOCKLEN)];

            //  BLOCK ENCRYPT
            AESEncryptBlock(data, hlp);
        }

        hlp += BLOCKLEN;

        //
        // PROCESS LAST BLOCK
        //
        if (bStrengthLastBlock) {
            // STRENGTHENED LAST BLOCK OPERATION BY KLIMA + ROSA (ftp://ftp.decros.cz/pub/Archiv/Publications/2002/cbc.pdf)
            // Cn = Encrypt(Decrypt(Pn) XOR Encrypt(Cn-1) XOR Cn-1)

            // DECRYPT [== Decrypt(Pn)]
            AESDecryptBlock(data, hlp);

            // GET PREVIOUS BLOCK
            if (numBlocks == 1) {
              // PREVIOUS IS IV
              Util.arrayCopyNonAtomic(m_IV, m_IVOffset, prevBlock, (short) 0, BLOCKLEN);
            }
            else {
              // PREVIOUS IS CIPHERTEXT
              Util.arrayCopyNonAtomic(data, (short) (hlp - BLOCKLEN), prevBlock, (short) 0, BLOCKLEN);
            }

            // ENCRYPT [== Encrypt(Cn-1)]
            AESEncryptBlock(prevBlock, (short) (0));

            // XOR Decrypt(Pn) WITH Encrypt(Cn-1)
            for (j = 0; j < BLOCKLEN; j++) data[(short) (j + hlp)] ^= prevBlock[j];
        }
        else {
            // NO SPECIAL LAST BLOCK PRE-ACTION, LAST BLOCK ENCRYPT IS CLASSIC (AND VULNERABLE) CBC OPERATION (SAME AS FOR PREVIOUS BLOCKS)
        }

        // XOR PREVIOUS CIPHER OUTPUT
        if (i > 0) for (j = 0; j < BLOCKLEN; j++) data[(short) (j + hlp)] ^= data[(short) (j + hlp - BLOCKLEN)];

        //  BLOCK ENCRYPT
        AESEncryptBlock(data, hlp);

        return dataLen;
    }

     /**
      * Decrypt buffer using CBC cipher mode, key schedule must be already processed
      * @param data
      * @param dataOffset
      * @param dataLen
      * @param bUsePadding - if TRUE, PKCS#5 padding is used, NO padding otherwise
      * @param bStrengthLastBlock - of TRUE, Vaudenay CBC-PADDING attack countermeasure is used
      * @return output data length
      */
    public short AESDecryptCBC(byte data[], short dataOffset, short dataLen, boolean bUsePadding, boolean bStrengthLastBlock) {
        byte  r;
        short i;
        byte  j;
        short numBlocks = (short) (dataLen / BLOCKLEN);
        short hlp = (short) (dataOffset + dataLen);
        byte  prevBlock[] = new byte[BLOCKLEN];

        if ((dataLen % BLOCKLEN) != 0) ISOException.throwIt(SecureAlgManager.SW_CIPHER_DATA_LENGTH_BAD);
        if (m_IV == null) ISOException.throwIt(SecureAlgManager.SW_IV_BAD);

        //
        // PROCESS LAST BLOCK
        //
        hlp -= BLOCKLEN;

        if (bStrengthLastBlock) {
            // STRENGTHENED LAST BLOCK OPERATION BY KLIMA + ROSA (ftp://ftp.decros.cz/pub/Archiv/Publications/2002/cbc.pdf)
            // Pn = Encrypt(Encrypt(Cn-1) XOR Cn-1 XOR Decrypt(Cn))

            // DECRYPT [== Decrypt(Cn)]
            AESDecryptBlock(data, hlp);

            // GET PREVIOUS BLOCK
            if (numBlocks == 1) {
              // PREVIOUS IS IV
              Util.arrayCopyNonAtomic(m_IV, m_IVOffset, prevBlock, (short) 0, BLOCKLEN);
            }
            else {
              // PREVIOUS IS CIPHERTEXT
              Util.arrayCopyNonAtomic(data, (short) (hlp - BLOCKLEN), prevBlock, (short) 0, BLOCKLEN);
            }

            // XOR Decrypt(Cn) WITH Cn-1
            for (j = 0; j < BLOCKLEN; j++) data[(short) (j + hlp)] ^= prevBlock[j];

            // ENCRYPT [== Encrypt(Cn-1)]
            AESEncryptBlock(prevBlock, (short) (0));

            // XOR Decrypt(Cn) WITH Encrypt(Cn-1)
            for (j = 0; j < BLOCKLEN; j++) data[(short) (j + hlp)] ^= prevBlock[j];

            // ENCRYPT LAST BLOCK
            AESEncryptBlock(data, hlp);
        }
        else {
            // NO SPECIAL ACTION, LAST BLOCK DECRYPT IS CLASSIC (AND POSSIBLE VULNERABLE) CBC OPERATION (SAME AS FOR PREVIOUS BLOCKS)

            //  BLOCK DECRYPT
            AESDecryptBlock(data, hlp);

            // REMOVE CBC XOR
            if (numBlocks == 1) {
              // XOR WITH INIT VECTOR
              for (j = 0; j < BLOCKLEN; j++) data[(short) (j + hlp)] ^= m_IV[(short) (j + m_IVOffset)];
            }
            else {
              // XOR WITH PREVIOUS CIPHERTEXT BLOCK
              for (j = 0; j < BLOCKLEN; j++) data[(short) (j + hlp)] ^= data[(short) (j + hlp - BLOCKLEN)];
            }
        }

        //
        // PROCESS ALL BLOCKS EXCEPT LAST (STANDARD CBC MODE)
        //
        for (i = (short) (numBlocks - 1); i > 0; i--) {
            hlp -= BLOCKLEN;

            //  BLOCK DECRYPT
            AESDecryptBlock(data, hlp);

            // REMOVE CBC XOR
            if (i == 1) {
              // XOR WITH INIT VECTOR
              for (j = 0; j < BLOCKLEN; j++) data[(short) (j + hlp)] ^= m_IV[(short) (j + m_IVOffset)];
            }
            else {
              // XOR WITH PREVIOUS CIPHERTEXT BLOCK
              for (j = 0; j < BLOCKLEN; j++) data[(short) (j + hlp)] ^= data[(short) (j + hlp - BLOCKLEN)];
            }
        }

        //
        //  REMOVE PADDING
        //
        if (bUsePadding) {
          // REMOVE PADING BY PKCS#5
          byte paddingByte = data[(short) (dataOffset + dataLen - 1)];
          if (paddingByte == 0) {
            // LAST BLOCK MUST BE ONLY ZEROS
            hlp = (short) (dataOffset + dataLen - BLOCKLEN);
            for (i = (short) (dataOffset + dataLen - 2); i >= hlp; i--) {
              if (data[i] != 0) ISOException.throwIt(SecureAlgManager.SW_CIPHER_DATA_LENGTH_BAD);
            }

            // SET OUTPUT DATA LENGTH
            dataLen -= BLOCKLEN;
          }
          else {
            if (paddingByte <= 0x0F) {
              // LAST paddingByte BYTES MUST BE ONLY 'paddingByte' VALUE
              hlp = (short) (dataOffset + dataLen - paddingByte);
              for (i = (short) (dataOffset + dataLen - 2); i >= hlp; i--) {
                if (data[i] != paddingByte) ISOException.throwIt(SecureAlgManager.SW_CIPHER_DATA_LENGTH_BAD);
              }

              // SET OUTPUT DATA LENGTH
              dataLen -= paddingByte;
            }
            else ISOException.throwIt(SecureAlgManager.SW_CIPHER_DATA_LENGTH_BAD);
          }
        }
        else {
          // NO PADDING USED
        }
        return dataLen;
    }
}