/*  SecureAlgProfile.java -- SecuredAlgorithm Profile class

    This is part of SecureFW project, protection interface for which allows controled usage of secured code.
    See http://www.svenda.com/petr/dpthesis/index.html for more information
    Copyright (C) 2004  Petr Svenda <petr@svenda.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please, report any bugs to author <petr@svenda.com>
/**/

package com.svenda.SecureAlg;
import javacard.framework.*;

public class SecureAlgProfile {
  private byte        m_uuid[];
  private short       m_lastLicenceOrdNum;
  protected short     m_allowedUsageCounter;

  public SecureAlgProfile(byte dataBuffer[], short offset, short dataLen) {
    m_uuid = new byte[SecureAlgManager.ALGORITHM_UUID_LENGTH];
    Util.arrayCopyNonAtomic(dataBuffer, offset, m_uuid, (short) 0, SecureAlgManager.ALGORITHM_UUID_LENGTH);
    m_lastLicenceOrdNum = 0;
    m_allowedUsageCounter = 0;
  }

  /**
   * Process input data of Secured Algorithm and return result. Result is copied
   * into dataBuffer
   * @param dataBuffer
   * @param offset
   * @param dataLen
   * @return length of result data
   */
  public short ProcessAlgData(byte dataBuffer[], short offset, short dataLen) {
    // EXAMPLE
    dataBuffer[offset] = 0x13;
    return 1;
  }

  /**
   * Check if is same algorithm by given UUID
   * @param uuidBuffer
   * @param offset
   * @param uuidLen
   * @return
   */
  public boolean IsAlgorithm(byte uuidBuffer[], short offset, short uuidLen) {
    boolean status = false;
    if (uuidLen == m_uuid.length) {
      if (Util.arrayCompare(m_uuid, (short) 0, uuidBuffer, offset, uuidLen) == 0) status = true;
    }
    return status;
  }

  /**
   * Get last accepted algorithm licence ordinary number for this algorithm
   * @return last accepted licence number
   */
  public short GetLastLicenceOrdNum() {
    return m_lastLicenceOrdNum;
  }

  /**
   * Update last accepted algorithm licence ordinary number
   * @param licenceOrdNum
   * @param outOfOrder
   * @return
   */
  public boolean UpdateLastLicenceOrdNum(short licenceOrdNum, boolean outOfOrder) {
    boolean status = false;
    // UPDATE ONLY TO HIGHER LICENCE NUMBERS
    if (m_lastLicenceOrdNum >= licenceOrdNum) status = false;
    else {
      if (((short) (m_lastLicenceOrdNum + 1) == licenceOrdNum) || outOfOrder) {
        m_lastLicenceOrdNum = licenceOrdNum;
        status = true;
      }
      else ISOException.throwIt(SecureAlgManager.SW_LICENCE_OUTOFORDER);
    }

    return status;
  }

  /**
   * Copy algorithm UUID
   * @param dataBuffer
   * @param offset
   * @return
   */
  public short GetAlgorithmUUID(byte dataBuffer[], short offset) {
    Util.arrayCopy(m_uuid, (short) 0, dataBuffer, offset, (short) m_uuid.length);
    return (short) m_uuid.length;
  }

  /**
   * Update algorithm allowed usage counter
   * @param usageCount
   */
  public void UpdateUsageCounter(short usageCount) {
    m_allowedUsageCounter += usageCount;
  }
}