/*  Stack.java -- JavaCard version of Java Stack class

    This is part of SecureFW project, protection interface for which allows controled usage of secured code.
    See http://www.svenda.com/petr/dpthesis/index.html for more information
    Copyright (C) 2004  Petr Svenda <petr@svenda.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please, report any bugs to author <petr@svenda.com>
/**/

package com.svenda.SecureAlg;
import javacard.framework.*;

public class Stack {
  private final static byte STACK_INCREMENT = 10;       // STACK RESIZE INCREMENT WHEN REACHING MAXIMUM
  private final static byte STACK_EMPTY = -1;           // STACK IS EMPTY, MOTHING TO POP
  private short  m_modeStackMax = 10;                   // CURRENT MAXIMAL STACK SIZE
  private byte[] m_modeStack = new byte[m_modeStackMax];// STACK VALUES
  private short  m_modeStackTop = -1;                   // POINTED TO TOP OF STACK

  public Stack() {}

  /**
   *  Push new value on top of stack
   *  @param value Value to push
   */
  public void push(byte value) {
    if (m_modeStackTop < m_modeStackMax) {
      // PUSH TO TOP
      m_modeStackTop++;
      m_modeStack[m_modeStackTop] = value;
    }
    else {
      // RESIZE STACK BUFFER
      m_modeStackMax += STACK_INCREMENT;
      byte[] m_newModeStack = new byte[m_modeStackMax];
      Util.arrayCopy(m_modeStack, (short) 0, m_newModeStack, (short) 0, (short) (m_modeStackTop + 1));
      m_modeStack = null;
      m_modeStack = m_newModeStack;

      // CALL AGAIN
      push(value);
    }
  }

  /**
   *  Pop stack topmost value
   */
  public byte pop() {
    byte result = STACK_EMPTY;

    if (m_modeStackTop > -1) {
      result = m_modeStack[m_modeStackTop];
      m_modeStackTop--;
    }

    return result;
  }

  /**
   *  Return stack topmost value
   */
  public byte top() {
    if (m_modeStackTop > -1) return m_modeStack[m_modeStackTop];
    else return STACK_EMPTY;
  }

  /**
   *  Return stack fullness state
   */
  public boolean empty() {
    if (m_modeStackTop > -1) return false;
    else return true;
  }

  /**
   *  Clear all values from stack, stack is empty after this operation
   */
  public void clear() {
    m_modeStackTop = (short) -1;
  }
}